use chrono::{Duration, Utc};
use jsonwebtoken::{errors::Error, DecodingKey, EncodingKey, Header, Validation};
use serde::{Deserialize, Serialize};

use crate::model::user::User;

#[derive(Debug, Serialize, Deserialize)]
pub struct Claims {
    pub exp: usize,
    pub uid: i32,
    pub role: String,
}

impl Claims {
    pub fn new(user: &User) -> Self {
        // Expire tokens after 90 days
        // TODO: give tokens an ID to expire them earlier if necessary? Or a salt that's changed on
        // password update
        let now = Utc::now() + Duration::days(90);

        Self {
            exp: now.timestamp() as usize,
            uid: user.id,
            role: user.role.into(),
        }
    }

    pub fn encode(self, key: &EncodingKey) -> Result<String, Error> {
        jsonwebtoken::encode(&Header::default(), &self, key)
    }

    pub fn decode(token: &str, key: &DecodingKey) -> Result<Self, Error> {
        let token = jsonwebtoken::decode::<Claims>(token, key, &Validation::default())?;

        Ok(token.claims)
    }
}
